@@ -70,6 +70,18 @@ typedef struct gps_pause_info {
 #define PAUSE_STATE_PAUSED 1
 #define PAUSE_STATE_BEHIND 2
 
+static __inline__ struct class *_ligo_class_create(const char* name)
+{
+#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 4, 0)
+    return class_create(THIS_MODULE, name);
+#else
+    return class_create(name);
+#endif
+}
+
+// some forward to reduce warnings
+int get_cur_time(unsigned long *req);
+
 // track if we have had a pause operation ever
 // this is reset on a reset.
 static atomic64_t gps_has_been_paused = ATOMIC_INIT(0);
@@ -652,7 +664,7 @@ static int __init symmetricom_init(void)
             goto out;
     }
 
-    if (IS_ERR(gpstime_class = class_create( THIS_MODULE, "ligo_gpstime" )))
+    if (IS_ERR(gpstime_class = _ligo_class_create( "ligo_gpstime" )))
     {
             printk(KERN_ERR "could not allocate device class for gpstime\n");
             goto out_unalloc_region;
