@@ -1,5 +1,10 @@
 //lptc specific sysfs definitions
 
+// fowards to reduce warnings
+int str_to_uint(const char *buf, u32 *value);
+const char *find_first_underscore(const char *buf);
+int get_slot_from_name(const char *name, long *slot);
+
 /// read a string from among several possible radices
 int str_to_uint(const char *buf, u32 *value) {
   int radix;
