@@ -38,11 +38,17 @@ of this distribution.
 
 #include <daqmap.h>
 #include <daq_data_types.h>
+#include <daq_core.h>
+#include <cds-pubsub/pub.hh>
+#include <crc.h>
+#include <sched.h>
+#include <pthread.h>
 
 #define BOOST_ASIO_USE_BOOST_DATE_TIME_FOR_SOCKET_IOSTREAM
 #include <boost/asio.hpp>
 #include <boost/date_time/posix_time/posix_time.hpp>
 #include <boost/lockfree/spsc_queue.hpp>
+#include <boost/algorithm/string/predicate.hpp>
 
 #include <rapidjson/writer.h>
 #include <rapidjson/stringbuffer.h>
@@ -58,8 +64,10 @@ of this distribution.
 #include "simpleargs.h"
 #include "simple_pv.h"
 #include "util/user/check_file_crc.h"
+#include "util/user/raii.hh"
 
 #include <iostream>
+#include <boost/serialization/vector.hpp>
 
 #define EDCU_MAX_CHANS 60000
 #define DIAG_QUEUE_DEPTH 3
@@ -330,13 +338,18 @@ int        nextTrig = 0;
 class EdcuClock
 {
 public:
-    EdcuClock( const char* sync_source, int wait_ms )
+    EdcuClock( std::string& sync_source, int wait_ms )
     {
-        if ( sync_source != nullptr && strcmp( sync_source, "-" ) != 0 )
+        if ( !sync_source.empty() && sync_source != "-" )
         {
+            std::string shmem_name = sync_source;
+            if ( !boost::algorithm::ends_with(shmem_name, "_daq") )
+            {
+                shmem_name += "_daq";
+            }
             sipc_shm_ptr_ = std::unique_ptr< shmem::shmem >
                             (
-                              new shmem::shmem(sync_source, DEFAULT_SHMEM_ALLOC_SIZE_MB)
+                              new shmem::shmem(shmem_name.c_str(  ), DEFAULT_SHMEM_ALLOC_SIZE_MB)
                             );
             auto sync_addr = sipc_shm_ptr_->mapping<char>();
 
@@ -1055,6 +1068,39 @@ edcuLoadSpecial( int index, int value )
     }
 }
 
+/*!
+ * @brief calculate the size of the data block needed
+ * @return the number of bytes to be transfered, or -1 on error
+ * @note -1 will only be returned if an unknown data type is found
+ */
+int
+calculateDaqDataLength( )
+{
+    int ii = 0;
+    int req_size = 0;
+
+    for ( ii = 0; ii < daqd_edcu1.num_chans; ii++ )
+    {
+        switch ( daqd_edcu1.channel_type[ ii ] )
+        {
+            case _16bit_integer:
+                req_size += sizeof(std::int16_t);
+                break;
+            case _32bit_integer:
+            case _32bit_float:
+                req_size += sizeof(float);
+                break;
+            case _64bit_double:
+                req_size += sizeof(double);
+                break;
+            default:
+                fprintf(stderr, "unexpected data type found %d\n", daqd_edcu1.channel_type[ ii ] );
+                return -1;
+        }
+    }
+    return req_size;
+}
+
 /*!
  * @brief copy the data from daqd_edcu1.channel_value to the output buffer
  * @param daqData The output buffer
@@ -1103,14 +1149,9 @@ copyDaqData( char* daqData )
     return static_cast< int >( daqData - dataStart );
 }
 
-// **************************************************************************
 void
-edcuWriteData( int daqBlockNum, unsigned long cycle_gps_time, int dcuId, shmem::shmem & daq_shmem)
-// **************************************************************************
+loadSpecialVars()
 {
-    char* daqData;
-    int   buf_size;
-
     edcuLoadSpecial( num_chans_index, daqd_edcu1.num_chans );
     edcuLoadSpecial( con_chans_index, daqd_edcu1.con_chans );
     edcuLoadSpecial( nocon_chans_index,
@@ -1118,6 +1159,17 @@ edcuWriteData( int daqBlockNum, unsigned long cycle_gps_time, int dcuId, shmem::
     edcuLoadSpecial( uptime_index, daqd_edcu1.uptime );
     edcuLoadSpecial( data_rate_index, daqd_edcu1.data_rate );
     edcuLoadSpecial( gpstime_index, static_cast< int >( daqd_edcu1.gpsTime ) );
+}
+
+// **************************************************************************
+void
+edcuWriteData( int daqBlockNum, unsigned long cycle_gps_time, int dcuId, int expectedDataSize, shmem::shmem & daq_shmem)
+// **************************************************************************
+{
+    char* daqData;
+    int   buf_size;
+
+    loadSpecialVars();
 
     buf_size = DAQ_DCU_BLOCK_SIZE;
     daqData = (char*)( (daq_shmem.mapping<char>() + CDS_DAQ_NET_DATA_OFFSET)
@@ -1147,6 +1199,41 @@ edcuWriteData( int daqBlockNum, unsigned long cycle_gps_time, int dcuId, shmem::
     dipc->cycle = daqBlockNum; // Triggers sending of data by mx_stream.
 }
 
+void
+edcuPublishData(int daqBlockNum, unsigned long cycle_gps_time, int dcuId, int expectedDataSize, pub_sub::Publisher& publisher)
+{
+    loadSpecialVars();
+
+    std::size_t reqSize = sizeof(daq_multi_dcu_header_t) + static_cast<std::size_t>(expectedDataSize);
+    std::shared_ptr<unsigned char[]> dataPtr(new unsigned char[reqSize]);
+
+    auto* header = reinterpret_cast<daq_multi_dcu_header_t*>(dataPtr.get());
+    char* daqData = reinterpret_cast<char*>( dataPtr.get() + sizeof(*header));
+
+    std::memset(header, 0, sizeof(*header));
+
+    copyDaqData( daqData );
+    auto crc = crc_ptr( daqData, expectedDataSize, 0 );
+    crc = crc_len(expectedDataSize , crc);
+
+    header->dcuTotalModels = 1;
+    header->fullDataBlockSize = expectedDataSize;
+    daq_msg_header_t& dcuHeader = header->dcuheader[0];
+    dcuHeader.dcuId = dcuId;
+    dcuHeader.fileCrc = daqFileCrc;
+    dcuHeader.status = 0;
+    dcuHeader.cycle = daqBlockNum;
+    dcuHeader.timeSec = cycle_gps_time;
+    dcuHeader.timeNSec = daqBlockNum * (1000000000/16);
+    dcuHeader.dataCrc = crc;
+    dcuHeader.dataBlockSize = expectedDataSize;
+    dcuHeader.tpBlockSize = 0;
+    dcuHeader.tpCount = 0;
+
+    pub_sub::KeyType key = (cycle_gps_time << 8) + daqBlockNum;
+    publisher.publish( key, pub_sub::Message( std::move(dataPtr), reqSize) );
+}
+
 
 std::pair< std::string, int >
 parse_address( const std::string& str )
@@ -1162,6 +1249,20 @@ parse_address( const std::string& str )
     return std::make_pair( std::move( hostname ), port );
 }
 
+void
+lock_this_thread()
+{
+    int index = sched_getcpu();
+    if (index < 0)
+    {
+        return;
+    }
+    cpu_set_t cpuset;
+    CPU_ZERO( &cpuset );
+    CPU_SET( index, &cpuset );
+    pthread_setaffinity_np( pthread_self() , sizeof(cpu_set_t), &cpuset );
+}
+
 /// Called on EPICS startup; This is generic EPICS provided function, modified
 /// for LIGO use.
 int
@@ -1183,6 +1284,7 @@ main( int argc, char* argv[] )
     const char* daqFile = nullptr;
     const char* listen_interface = nullptr;
     const char* sync_source = nullptr;
+    const char* _publish_str = nullptr;
     int         delay_ms = 0;
 
     memset( (void*)&daqd_edcu1, 0, sizeof( daqd_edcu1 ) );
@@ -1274,6 +1376,13 @@ main( int argc, char* argv[] )
                          "use the gpstime driver",
                          &sync_source,
                          "-" );
+    args_add_string_ptr( arg_parser,
+                         ARGS_NO_SHORT,
+                         "publish",
+                         "",
+                         "Optional pub/sub publish string, if empty write data to shmem",
+                         &_publish_str,
+                         "");
 
     if ( args_parse( arg_parser, argc, argv ) < 0 )
     {
@@ -1290,6 +1399,7 @@ main( int argc, char* argv[] )
         std::cout << "Delays are ignored when syncing from an iop" << std::endl;
         delay_ms = 0;
     }
+    std::string publish_str{_publish_str};
 
     InternalChanNames internal_channels( daqd_edcu1.prefix );
     diag_args.address = parse_address( listen_interface );
@@ -1302,10 +1412,23 @@ main( int argc, char* argv[] )
 
     std::string model_name(sync_source);
     std::transform(model_name.begin(), model_name.end(), model_name.begin(), ::tolower);
-	EdcuClock clock( model_name.c_str(), delay_ms );
+    EdcuClock clock( model_name, delay_ms );
     std::string shmmem_name(daqsharedmemname);
     std::transform(shmmem_name.begin(), shmmem_name.end(), shmmem_name.begin(), ::tolower);
-	shmem::shmem daq_shmem = shmem::shmem(shmmem_name + "_daq", DEFAULT_SHMEM_ALLOC_SIZE_MB);
+
+    std::unique_ptr<pub_sub::Publisher> publisher{nullptr};
+    shmem::shmem daq_shmem{};
+    if (publish_str.empty( ))
+    {
+        std::cout << "Selecting shmem output" << std::endl;
+        daq_shmem = shmem::shmem(shmmem_name + "_daq", DEFAULT_SHMEM_ALLOC_SIZE_MB);
+    }
+    else
+    {
+        std::cout << "Selecting pub/sub output" << std::endl;
+        publisher = raii::make_unique_ptr<pub_sub::Publisher>();
+        publisher->add_destination( publish_str );
+    }
 
 
     edcuCreateChanList( daqd_edcu1, daqFile, &daqFileCrc, internal_channels );
@@ -1355,6 +1478,15 @@ main( int argc, char* argv[] )
     std::thread diag_thread(
         diag_thread_mainloop, &diag_args, &internal_channels );
 
+    int expected_data_size = calculateDaqDataLength(  );
+    if ( expected_data_size < 0 )
+    {
+        fprintf(stderr, "Invalid data type found in the edcu.ini");
+        exit(1);
+    }
+
+    lock_this_thread();
+
     // Start Infinite Loop
     // *******************************************************************************
     for ( ;; )
@@ -1372,10 +1504,23 @@ main( int argc, char* argv[] )
         daqd_edcu1.gpsTime = now.sec;
         daqd_edcu1.epicsSync = clock.cycle( );
 
-        edcuWriteData(daqd_edcu1.epicsSync, 
-                      daqd_edcu1.gpsTime, 
-                      daqd_edcu1.dcuid, 
-                      daq_shmem );
+        if ( publisher )
+        {
+            edcuPublishData(daqd_edcu1.epicsSync,
+                            daqd_edcu1.gpsTime,
+                            daqd_edcu1.dcuid,
+                            expected_data_size,
+                            *publisher );
+        }
+        else
+        {
+            edcuWriteData(daqd_edcu1.epicsSync,
+                          daqd_edcu1.gpsTime,
+                          daqd_edcu1.dcuid,
+                          expected_data_size,
+                          daq_shmem );
+        }
+
 
         transmit_time = transmit_time + time_step;
 
