@@ -58,6 +58,9 @@
 #include <linux/delay.h> //udelay()
 #include <asm/cacheflush.h>
 
+// Forward Declaration
+void fe_start_controller( void );
+
 #ifdef NO_CPU_SHUTDOWN
 #include <linux/kthread.h> //kthread_should_stop()
 #endif
