@@ -159,7 +159,9 @@ static char *versionId = "Version $Id$" ;
       /* determine network address client address */
       {
          struct sockaddr_in*	addr;
-         addr = svc_getcaller ((SVCXPRT*) xprt);
+	 // FIXME/NOTE this is implemented as returning a sockaddr_in6*
+	 // it may break in the future if we use it for more.
+         addr = (struct sockaddr_in*)svc_getcaller ((SVCXPRT*) xprt);
          *clntaddr = addr->sin_addr;
       }
 
