@@ -44,6 +44,8 @@
 static double last_dac_out_cache[MAX_IO_MODULES][MAX_DAC_CHN_PER_MOD];
 static double last_tx_was_good[MAX_IO_MODULES][MAX_DAC_CHN_PER_MOD] = {{0,},{0,},};
 
+// Forward declarations
+int preload_check(int card);
 
 int
 iop_dac_init( int errorPend[] )
