@@ -46,8 +46,8 @@
 
 #include "simple_pv.h"
 #include "recv_buffer.hh"
-#include "make_unique.hh"
 #include "arena.hh"
+#include "util/user/raii.hh"
 
 #define __CDECL
 
@@ -881,7 +881,7 @@ main( int argc, char** argv )
     subscribers.reserve( subscription_strings.size( ) );
     if ( !thread_per_sub )
     {
-        subscribers.emplace_back( make_unique_ptr< pub_sub::Subscriber >( ) );
+        subscribers.emplace_back( raii::make_unique_ptr< pub_sub::Subscriber >( ) );
         load_sub_plugins( *subscribers.back( ).get( ), sub_plugins );
     }
     for ( const auto& conn_str : subscription_strings )
@@ -889,7 +889,7 @@ main( int argc, char** argv )
         if ( thread_per_sub )
         {
             subscribers.emplace_back(
-                make_unique_ptr< pub_sub::Subscriber >( ) );
+                raii::make_unique_ptr< pub_sub::Subscriber >( ) );
             load_sub_plugins( *subscribers.back( ).get( ), sub_plugins );
         }
         subscribers.front( )->SetupDebugHooks( &debug );
