@@ -17,8 +17,6 @@
 
 #include <boost/algorithm/string.hpp>
 
-#include "make_unique.hh"
-
 #include "daqmap.h"
 #include "drv/fb.h"
 #include "daq_core.h"
@@ -33,6 +31,62 @@ namespace cps_plugins
 {
     namespace detail
     {
+
+
+        DcuRate
+        lookup_dcu_and_rate(const std::string& name)
+        {
+            int rate = 16;
+            int dcu_id = 0;
+
+            auto idx = name.find(":");
+            if ( idx != std::string::npos )
+            {
+                auto model_name = name.substr( 0, idx );
+                auto dcu_str = name.substr( idx + 1 );
+                idx = dcu_str.find(":");
+
+                if (idx != std::string::npos)
+                {
+                    auto rate_str = dcu_str.substr( idx + 1 );
+                    dcu_str = dcu_str.substr( 0, idx);
+                    rate = std::atoi( rate_str.c_str() );
+                    if ( rate == 0 )
+                    {
+                        std::ostringstream os{};
+                        os << "Unable to parse the data rate of " << name;
+                        throw std::runtime_error( os.str() );
+                    }
+                }
+                dcu_id = std::stoi( dcu_str );
+                if (dcu_id == 0)
+                {
+                    {
+                        std::ostringstream os{};
+                        os << "Unable to parse the dcu_id of " << name;
+                        throw std::runtime_error( os.str() );
+                    }
+                }
+                return DcuRate{
+                    model_name,
+                    dcu_id,
+                    rate,
+                };
+            }
+            int status = get_model_rate_dcuid( &rate, &dcu_id, name.c_str(  ), nullptr);
+            if ( status != 0 || rate == 0 )
+            {
+                std::ostringstream os{};
+                os << "Unable to parse the dcu_id of " << name;
+                throw std::runtime_error( os.str() );
+            }
+            return DcuRate{
+                name,
+                dcu_id,
+                rate,
+            };
+        }
+
         constexpr const int rmipc_max_subs = 128;
         class RmIpcSub : public pub_sub::plugins::Subscription
         {
@@ -78,7 +132,12 @@ namespace cps_plugins
                 }
                 for ( const auto& name : mbuf_names )
                 {
-                    std::string shmem_fname = name + "_daq";
+                    auto dcu_rate_info = lookup_dcu_and_rate( name );
+
+                    dcuid[ i ] = dcu_rate_info.dcu_id;
+                    modelrates[ i ] = dcu_rate_info.rate;
+
+                    std::string shmem_fname = dcu_rate_info.name + "_daq";
                     shmHandles[ i ] =
                         shmem::shmem( shmem_fname, FE_MBUF_SIZE_MB );
                     volatile void* dcu_addr =
@@ -97,15 +156,6 @@ namespace cps_plugins
                         (struct
                          cdsDaqNetGdsTpNum*)( (char*)dcu_addr +
                                               CDS_DAQ_NET_GDS_TP_TABLE_OFFSET );
-
-                    auto status = get_model_rate_dcuid(
-                        &modelrates[ i ], &dcuid[ i ], name.c_str( ), nullptr );
-                    if ( status != 0 || modelrates[ i ] == 0 )
-                    {
-                        std::ostringstream os;
-                        os << "Unable to get the modelrate of " << name;
-                        throw std::runtime_error( os.str( ) );
-                    }
                     ++i;
                 }
                 th_ = std::thread( [this]( ) { rmipc_sub_loop( ); } );
