@@ -35,14 +35,16 @@
 #include <boost/algorithm/string.hpp>
 #include <boost/optional.hpp>
 #include <cds-pubsub/sub.hh>
+#include "sub_plugin_daq_m.hh"
+#include "sub_plugin_rmipc.hh"
 #include "cps_recv_admin.hh"
 
 #include "simpleargs.h"
 
 #include "simple_pv.h"
 #include "recv_buffer.hh"
-#include "make_unique.hh"
 #include "dc_stats.hh"
+#include "util/user/raii.hh"
 
 #include "message_filter.hh"
 #include "cds-shmem.h"
@@ -314,7 +316,7 @@ public:
         if ( next_stage_ )
         {
             next_stage_->emplace(
-                make_unique_ptr< daq_dc_data_t >( input_data ) );
+                raii::make_unique_ptr< daq_dc_data_t >( input_data ) );
         }
     }
 
@@ -936,33 +938,48 @@ main( int argc, char** argv )
     std::vector< cps_admin::SubEntry >                    admin_sub_entries{ };
     subscribers.reserve( subscription_strings.size( ) );
 
-    if ( !thread_per_sub )
     {
-        subscribers.emplace_back( make_unique_ptr< pub_sub::Subscriber >( ) );
-    }
-    for ( const auto& conn_str : subscription_strings )
-    {
-        if ( thread_per_sub )
+        std::vector<std::shared_ptr<pub_sub::plugins::SubscriptionPluginApi>> plugins{};
+        plugins.push_back(std::make_shared<cps_plugins::SubPluginDaqMApi>());
+        plugins.push_back(std::make_shared<cps_plugins::SubPluginRmIpcApi>());
+
+        if ( !thread_per_sub )
         {
-            subscribers.emplace_back(
-                make_unique_ptr< pub_sub::Subscriber >( ) );
+            subscribers.emplace_back( raii::make_unique_ptr< pub_sub::Subscriber >( ) );
+            for ( auto const& plugin : plugins)
+            {
+                subscribers.back( )->load_plugin( plugin );
+            }
         }
-        subscribers.back( )->SetupDebugHooks( &debug );
-        fprintf( stderr, "Beginning subscription on %s\n", conn_str.c_str( ) );
-        auto sub_id = subscribers.back( )->subscribe(
-            conn_str, [ &message_filters ]( pub_sub::SubMessage sub_msg ) {
-                for ( const auto& filter : message_filters )
+        for ( const auto& conn_str : subscription_strings )
+        {
+            if ( thread_per_sub )
+            {
+                subscribers.emplace_back(
+                    raii::make_unique_ptr< pub_sub::Subscriber >( ) );
+                for ( auto const& plugin : plugins)
                 {
-                    if ( !( *filter )( sub_msg ) )
-                    {
-                        return;
-                    }
+                    subscribers.back( )->load_plugin( plugin );
                 }
-                circular_buffer.ingest(
-                    *( (daq_multi_dcu_data_t*)sub_msg.data( ) ) );
-            } );
-        admin_sub_entries.emplace_back( cps_admin::SubEntry{
-            conn_str, sub_id, subscribers.back( ).get( ) } );
+            }
+            subscribers.back( )->SetupDebugHooks( &debug );
+
+            fprintf( stderr, "Beginning subscription on %s\n", conn_str.c_str( ) );
+            auto sub_id = subscribers.back( )->subscribe(
+                conn_str, [ &message_filters ]( pub_sub::SubMessage sub_msg ) {
+                    for ( const auto& filter : message_filters )
+                    {
+                        if ( !( *filter )( sub_msg ) )
+                        {
+                            return;
+                        }
+                    }
+                    circular_buffer.ingest(
+                        *( (daq_multi_dcu_data_t*)sub_msg.data( ) ) );
+                } );
+            admin_sub_entries.emplace_back( cps_admin::SubEntry{
+                conn_str, sub_id, subscribers.back( ).get( ) } );
+        }
     }
 
     boost::optional< cps_admin::AdminInterface > admin_instance{ boost::none };
